@@ -142,10 +142,10 @@ class Biso:
     )
     default_main_color = "blue"
     default_margin = dict(
-        l=5,
-        r=5,
-        b=5,
-        t=5,
+        l=15,
+        r=15,
+        b=15,
+        t=15,
         pad=4
     )
     default_max_entities = 1000 # default_max_requested_works
@@ -313,11 +313,41 @@ class Biso:
             self.info += r"\\"
         if (self.n_entities_found is not None and self.n_entities_found > self.max_plotted_entities and
                 not hide_n_entities_warning):
-            self.info += self._("The number of displayed entities was limited to ")
+            if self.__class__.__name__ == "AnrProjects" or self.__class__.__name__ == "EuropeanProjects":
+                self.info += self._(f"The number of displayed projects was limited to ")
+            elif self.__class__.__name__ == "Chapters":
+                self.info += self._(f"The number of displayed chapters was limited to ")
+            elif (self.__class__.__name__ == "CollaborationNames" or
+                    self.__class__.__name__ == "PrivateSectorCollaborations"):
+                self.info += self._(f"The number of displayed collaborations was limited to ")
+            elif self.__class__.__name__ == "Conferences":
+                self.info += self._(f"The number of displayed conferences was limited to ")
+            elif self.__class__.__name__ == "Journals" or self.__class__.__name__ == "JournalsHal":
+                self.info += self._(f"The number of displayed journals was limited to ")
+            elif self.__class__.__name__ == "WorksBibtex":
+                self.info += self._(f"The number of displayed works was limited to ")
+            elif self.__class__.__name__ == "WorksType":
+                self.info += self._(f"The number of displayed work types was limited to ")
+            else:
+                self.info += self._(f"The number of displayed entities was limited to ")
             self.info += str(self.max_plotted_entities)
             self.info += self._(". In the API, ")
             self.info += str(self.n_entities_found)
-            self.info += self._(" entities were found.")
+            if self.__class__.__name__ == "AnrProjects" or self.__class__.__name__ == "EuropeanProjects":
+                self.info += self._(f" projects were found.")
+            elif self.__class__.__name__ == "Chapters":
+                self.info += self._(f" chapters were found.")
+            elif (self.__class__.__name__ == "CollaborationNames" or
+                    self.__class__.__name__ == "PrivateSectorCollaborations"):
+                self.info += self._(f" collaborations were found.")
+            elif self.__class__.__name__ == "Conferences":
+                self.info += self._(f" conferences were found.")
+            elif self.__class__.__name__ == "Journals" or self.__class__.__name__ == "JournalsHal":
+                self.info += self._(f" journals were found.")
+            elif self.__class__.__name__ == "WorksType":
+                self.info += self._(f" work types were found.")
+            else:
+                self.info += self._(" entities were found.")
             self.info += r"\\"
 
 
@@ -795,13 +825,16 @@ class AnrProjects(Biso):
         super().__init__(lab, year, **kwargs)
 
 
-    def fetch_data(self):
+    def fetch_data(self) -> dict[str, Any]:
         """
         Fetch data about ANR projects from the HAL API.
 
         This method queries the API to get the list of ANR projects and their counts.
         The data is stored in the `data` attribute as a dictionary where keys are ANR project acronyms
         and values are their respective counts.
+
+        :return: The info about the fetched data.
+        :rtype: dict[str, Any]
         """
         try:
             facet_url = (
@@ -832,6 +865,8 @@ class AnrProjects(Biso):
 class Chapters(Biso):
     """
     A class to fetch and generate a table of book chapters.
+
+    :cvar figure_file_extension: The file extension for the figures ("tex" for LaTeX file).
     """
 
     figure_file_extension = "tex"
@@ -851,13 +886,16 @@ class Chapters(Biso):
         super().__init__(lab, year, **kwargs)
 
 
-    def fetch_data(self):
+    def fetch_data(self) -> dict[str, Any]:
         """
         Fetch data about book chapters from the HAL API.
 
         This method queries the API to get the list of book chapters and their metadata.
         The data is stored in the `data` attribute as a pandas DataFrame with columns for title (`title_s`),
         book title (`bookTitle_s`), and publisher (`publisher_s`).
+
+        :return: The info about the fetched data.
+        :rtype: dict[str, Any]
         """
         try:
             url = (
@@ -891,7 +929,8 @@ class Chapters(Biso):
                     "publisher_s": self._("Publisher"),
                 })
                 self.data_status = DataStatus.OK
-            self.generate_plot_info()
+            # hide_n_entities_warning as this information is already displayed on the table
+            self.generate_plot_info(hide_n_entities_warning=True)
             return {"info": self.info}
         except Exception as e:
             print(f"Error fetching or formatting data: {e}")
@@ -1054,12 +1093,15 @@ class CollaborationMap(Biso):
             self.zoom_lon_range = zoom_lon_range
 
 
-    def fetch_data(self):
+    def fetch_data(self) -> dict[str, Any]:
         """
         Fetch data about collaborations from the HAL API and OpenAlex API.
 
         This method queries the API to get the list of collaborations and their metadata.
         It processes the data to create a DataFrame with latitude, longitude, and other relevant information.
+
+        :return: The info about the fetched data.
+        :rtype: dict[str, Any]
         """
         try:
             # get the list of DOI from HAL:
@@ -1295,13 +1337,16 @@ class CollaborationNames(Biso):
         super().__init__(lab, year, **kwargs)
 
 
-    def fetch_data(self):
+    def fetch_data(self) -> dict[str, Any]:
         """
         Fetch data about collaboration names from the HAL API only.
 
         This method queries the API to get the list of collaboration names and their counts.
         It processes the data to create a dictionary where keys are formatted structure names (including country flags)
         and values are their respective counts.
+
+        :return: The info about the fetched data.
+        :rtype: dict[str, Any]
         """
         def format_structure_name(struct_name: str, country_code: str) -> str:
             """
@@ -1331,12 +1376,12 @@ class CollaborationNames(Biso):
             structs_facet_url = (
                 f"https://api.archives-ouvertes.fr/search/{self.lab}/?q=publicationDateY_i:{self.year} AND "
                 f"docType_s:(ART OR COMM)&wt=json&rows=0&facet=true&facet.field=structId_i&"
-                f"facet.limit=10000&facet.mincount=1&stats=true&"
-                "stats.field={!count=true+cardinality=true}structId_i"
+                f"facet.limit=10000&facet.mincount=1"
             )
             res = requests.get(structs_facet_url).json()
-            self.n_entities_found = (res.get('stats', {}).get('stats_fields', {}).get('structId_i', {}).
-                                     get('cardinality', 0))
+            # We don't set the self.n_entities_found value as calculating it would request too many API requests.
+            # It would require the full iterations for the 10k structures in the for loop below, and it would not give
+            # the right value for collections with more than 10k collaborations.
             structs_id_facets = res.get('facet_counts', {}).get('facet_fields', {}).get('structId_i', [])
             if not structs_id_facets:
                 self.data_status = DataStatus.NO_DATA
@@ -1352,7 +1397,7 @@ class CollaborationNames(Biso):
                 if not self.countries_to_exclude:
                     facet_url = (
                     f"https://api.archives-ouvertes.fr/ref/structure/?q="
-                    f"docid:({" OR ".join(list(structs_id_count.keys())[i:i+500])}) AND "
+                    f"docid:({" OR ".join(list(structs_id_count.keys())[i:i+500])})&"
                     f"fl=docid,label_s,country_s&rows=10000"
                 )
                 else:
@@ -1370,6 +1415,9 @@ class CollaborationNames(Biso):
                     struct for struct in facets_res if struct['docid'] in list(structs_id_count.keys())[i:i+500]
                                                        and struct.get('country_s') not in self.countries_to_exclude
                 ]
+                # if we found more structures than the number to plot, stop here to save useless API requests
+                if len(struct_list) >= self.max_plotted_entities:
+                    break
             if not struct_list:
                 self.data_status = DataStatus.NO_DATA
                 self.generate_plot_info()
@@ -1415,13 +1463,16 @@ class Conferences(Biso):
         """
         super().__init__(lab, year, **kwargs)
 
-    def fetch_data(self):
+    def fetch_data(self) -> dict[str, Any]:
         """
         Fetch data about conferences from the HAL API.
 
         This method queries the API to get the list of conferences and their counts.
         It processes the data to create a dictionary where keys are formatted conference names (including country flags)
         and values are their respective counts.
+
+        :return: The info about the fetched data.
+        :rtype: dict[str, Any]
         """
         def format_conference_name(conf_name: str, country_code: str | None) -> str:
             """
@@ -1508,13 +1559,16 @@ class EuropeanProjects(Biso):
         super().__init__(lab, year, **kwargs)
 
 
-    def fetch_data(self):
+    def fetch_data(self) -> dict[str, Any]:
         """
         Fetch data about European projects from the HAL API.
 
         This method queries the API to get the list of European projects and their counts.
         The data is stored in the `data` attribute as a dictionary where keys are European project acronyms
         and values are their respective counts.
+
+        :return: The info about the fetched data.
+        :rtype: dict[str, Any]
         """
         try:
             facet_url=(
@@ -1545,6 +1599,8 @@ class EuropeanProjects(Biso):
 class Journals(Biso):
     """
     A class to fetch and generate a table of journals.
+
+    :cvar figure_file_extension: The file extension for the figures ("tex" for LaTeX file).
     """
 
     figure_file_extension = "tex"
@@ -1562,9 +1618,12 @@ class Journals(Biso):
         """
         super().__init__(lab, year, **kwargs)
 
-    def fetch_data(self):
+    def fetch_data(self) -> dict[str, Any]:
         """
         Fetch data about journals from the API.
+
+        :return: The info about the fetched data.
+        :rtype: dict[str, Any]
         """
         def format_bso_currency(currency):
             if currency == 'USD':
@@ -1836,9 +1895,12 @@ class JournalsHal(Biso):
         super().__init__(lab, year, **kwargs)
 
 
-    def fetch_data(self):
+    def fetch_data(self) -> dict[str, Any]:
         """
         Fetch data about Journals from the HAL API.
+
+        :return: The info about the fetched data.
+        :rtype: dict[str, Any]
         """
         try:
             facet_url = (
@@ -1928,12 +1990,15 @@ class OpenAccessWorks(Biso):
         self.year_range = year_range
 
 
-    def fetch_data(self):
+    def fetch_data(self) -> dict[str, Any]:
         """
         Fetch data about open access works from the HAL API.
 
         This method queries the API to get the count of open access works for each year in the specified year range.
         The data is stored in the `data` attribute as a pandas DataFrame with counts for different open access statuses.
+
+        :return: The info about the fetched data.
+        :rtype: dict[str, Any]
         """
         try:
             self.data=pd.DataFrame(
@@ -2115,9 +2180,12 @@ class PrivateSectorCollaborations(Biso):
         """
         super().__init__(lab, year, **kwargs)
 
-    def fetch_data(self):
+    def fetch_data(self) -> dict[str, Any]:
         """
         Fetch data about PrivateSectorCollaborations from the API.
+
+        :return: The info about the fetched data.
+        :rtype: dict[str, Any]
         """
         try:
             if self.scanr_api_url is None:
@@ -2173,6 +2241,66 @@ class PrivateSectorCollaborations(Biso):
             return {"info": self._("Error")}
 
 
+class WorksBibtex(Biso):
+    """
+    A class to fetch the works of a HAL collection and create the bibtex string.
+
+    :cvar figure_file_extension: The file extension for the figures ("bib" for LaTeX bibtex file).
+    """
+
+    figure_file_extension = "bib"
+
+    def fetch_data(self) -> dict[str, Any]:
+        """
+        Fetch the bibtex data from HAL.
+
+        :return: The info about the fetched data.
+        :rtype: dict[str, Any]
+        """
+        logging.info("Fetching HAL bibtex data...")
+        try:
+            url = (f"https://api.archives-ouvertes.fr/search/{self.lab}/?q=publicationDateY_i:{self.year}&"
+                   f"wt=json&rows=0")
+            self.n_entities_found = requests.get(url).json()["response"]["numFound"]
+            if self.max_plotted_entities > 10000:
+                logging.warning(
+                    f"Max entities set to {self.max_plotted_entities}, but HAL API only allows up to 10000."
+                    f"Setting self.max_plotted_entities to 10k and getting only 10k bibtex elements"
+                )
+                self.max_plotted_entities = 10000
+            url = (f"https://api.archives-ouvertes.fr/search/{self.lab}/?q=publicationDateY_i:{self.year}&"
+                   f"wt=bibtex&rows={self.max_plotted_entities}")
+            self.data = requests.get(url).text
+            if not self.data:
+                self.data_status = DataStatus.NO_DATA
+            else:
+                self.data_status = DataStatus.OK
+            self.generate_plot_info()
+            return {"info": self.info}
+        except Exception as e:
+            logging.error(f"Error fetching HAL bibtex data: {e}")
+            traceback.print_exc()
+            self.data = None
+            self.data_status = DataStatus.ERROR
+            return {"info": self._("Error")}
+
+
+    def get_figure(self) -> str:
+        """
+        Generate a LaTeX bibtex file.
+
+        :return: LaTeX bibtex string with all the references of the HAL collection.
+        :rtype: str
+        """
+        if self.data_status == DataStatus.NOT_FETCHED:
+            self.fetch_data()
+        if self.data_status == DataStatus.NO_DATA:
+            return "% " + self._(self._("No data"))
+        if self.data_status == DataStatus.ERROR:
+            return "% " + self._(self._("Error while making the bibliography"))
+        return self.data
+
+
 class WorksType(Biso):
     """
     A class to fetch and plot data about work types.
@@ -2192,13 +2320,16 @@ class WorksType(Biso):
         super().__init__(lab, year, **kwargs)
 
 
-    def fetch_data(self):
+    def fetch_data(self) -> dict[str, Any]:
         """
         Fetch data about work types from the HAL API.
 
         This method queries the API to get the list of work types and their counts.
         It processes the data to create a dictionary where keys are work type names and values are their respective
         counts.
+
+        :return: The info about the fetched data.
+        :rtype: dict[str, Any]
         """
         try:
             stats_url = (
